Package: CytoDx
Type: Package
Title: Robust prediction of clinical outcomes using cytometry data
        without cell gating
Version: 1.33.0
Author: Zicheng Hu
Maintainer: Zicheng Hu <zicheng.hu@ucsf.edu>
Description: This package provides functions that predict clinical outcomes using single cell data (such as flow cytometry data, RNA single cell sequencing data) without the requirement of cell gating or clustering. 
License: GPL-2
Encoding: UTF-8
Imports: doParallel, dplyr, glmnet, rpart, rpart.plot, stats,
        flowCore,grDevices, graphics, utils
Depends: R (>= 3.5)
Suggests: knitr, rmarkdown
VignetteBuilder: knitr, rmarkdown
RoxygenNote: 6.1.0
biocViews: ImmunoOncology, CellBiology, FlowCytometry,
        StatisticalMethod, Software, CellBasedAssays, Regression,
        Classification, Survival
git_url: https://git.bioconductor.org/packages/CytoDx
git_branch: devel
git_last_commit: 0f9a681
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:06:36 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(CytoDx.fit)
export(CytoDx.pred)
export(fcs2DF)
export(meanUnique)
export(pRank)
export(rank.ub.average)
export(set2DF)
export(treeGate)
import(grDevices)
import(graphics)
importFrom(doParallel,registerDoParallel)
importFrom(dplyr,"%>%")
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate_all)
importFrom(dplyr,summarise_all)
importFrom(flowCore,arcsinhTransform)
importFrom(flowCore,colnames)
importFrom(flowCore,compensate)
importFrom(flowCore,fsApply)
importFrom(flowCore,keyword)
importFrom(flowCore,logTransform)
importFrom(flowCore,read.flowSet)
importFrom(flowCore,transform)
importFrom(flowCore,transformList)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(rpart,rpart)
importFrom(rpart,rpart.control)
importFrom(rpart.plot,rpart.plot)
importFrom(stats,predict)
importFrom(utils,read.csv)

## Introduction

CytoDx is a method that predicts clinical outcomes using single cell data without the need of cell gating. It first predicts the association between each cell and the outcome using a linear statistical model. The cell level predictions are then averaged within each sample to represent the sample level predictor. A second sample level model is used to make prediction at the sample level.  


## Installation

To install the MetaCyto package, please run the following code:
```
library("devtools")
install_github("hzc363/CytoDx")
```

## Example
Here we provide a simple simulated example. Please see the vignette for an example that applies tssm to diagnos accute myeloid lymphoma using flow cytometry data. 

```
library(CytoDx)

# simulate 10 samples of class A 
A = data.frame("marker1"=c(rnorm(1000),rnorm(1000,mean=10)),
                "marker2"=c(rnorm(1000),rnorm(1000,mean=10)),
                "sample"=sample(1:10,2000,replace = T),
                "y"="A")

# simulate 10 samples of class B
B = data.frame("marker1"=c(rnorm(1500),rnorm(500,mean=10)),
               "marker2"=c(rnorm(1500),rnorm(500,mean=10)),
               "sample"=sample(11:20,2000,replace = T),
               "y"="B")             

# build CytoDx model
dat = rbind(A,B)
fit = CytoDx.fit(x = as.matrix(dat[,1:2]),
               y = (dat$y=="A"),
               xSample = dat$sample,
               family = "binomial",
               reg = F)

# predict 
pred = CytoDx.pred(fit = fit,
                 xNew = as.matrix(dat[,1:2]),
                 xSampleNew = dat$sample)

# plot probability
result = data.frame("Truth"=rep(c("A","B"),each=10),
                    "Prob"=pred$xNew.Pred.sample$y.Pred.s0)
stripchart(result$Prob~result$Truth, jitter = 0.1,
           vertical = TRUE, method = "jitter", pch = 20,
           xlab="Truth",ylab="Predicted Prob of A")
```

## ----out.width = "500px",echo=FALSE-------------------------------------------
knitr::include_graphics("tssm_intro.jpg")

## ----eval=FALSE---------------------------------------------------------------
# BiocManager::install("CytoDx")

## ----eval=FALSE---------------------------------------------------------------
# devtools::install_github("hzc363/CytoDx")

## ----results='asis',message=FALSE---------------------------------------------
library(CytoDx)

# Find data in CytoDx package
path <- system.file("extdata",package="CytoDx")

# read the ground truth
fcs_info  <- read.csv(file.path(path,"fcs_info.csv"))

# print out the ground truth
knitr::kable(fcs_info)

## ----results='asis',message=FALSE---------------------------------------------
# Find the training data
train_info <- subset(fcs_info,fcs_info$dataset=="train")

# Specify the path to the cytometry files
fn <- file.path(path,train_info$fcsName)

# Read cytometry files using fcs2DF function
train_data <- fcs2DF(fcsFiles=fn,
                    y=train_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))

## ----results='asis',message=FALSE---------------------------------------------
# Perfroms rank transformation
x_train <- pRank(x=train_data[,1:7],xSample=train_data$xSample)

# Convert data frame into matrix. Here we included the 2-way interactions.
x_train <- model.matrix(~.*.,x_train)

## ----results='asis',message=FALSE,warning=FALSE-------------------------------
# Build predictive model using the CytoDx.fit function
fit <- CytoDx.fit(x=x_train,
               y=(train_data$y=="aml"),
               xSample=train_data$xSample,
               family = "binomial",
               reg = FALSE)

## ----results='asis',message=FALSE---------------------------------------------
# Find testing data
test_info <- subset(fcs_info,fcs_info$dataset=="test")

# Specify the path to cytometry files
fn <- file.path(path,test_info$fcsName)

# Read cytometry files using fcs2DF function
test_data <- fcs2DF(fcsFiles=fn,
                    y=NULL,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
# Perfroms rank transformation
x_test <- pRank(x=test_data[,1:7],xSample=test_data$xSample)

# Convert data frame into matrix. Here we included the 2-way interactions.
x_test <- model.matrix(~.*.,x_test)

## ----results='asis',message=FALSE---------------------------------------------
# Predict AML using CytoDx.ped function
pred <- CytoDx.pred(fit,xNew=x_test,xSampleNew=test_data$xSample)

## ----results='asis',message=FALSE,fig.width = 5-------------------------------
# Cmbine prediction and truth
result <- data.frame("Truth"=test_info$Label,
                    "Prob"=pred$xNew.Pred.sample$y.Pred.s0)

# Plot the prediction
stripchart(result$Prob~result$Truth, jitter = 0.1,
           vertical = TRUE, method = "jitter", pch = 20,
           xlab="Truth",ylab="Predicted Prob of AML")

## ----results='asis',message=FALSE,fig.width = 5-------------------------------
# Use decision tree to find the cell subsets that are associated the AML.
TG <- treeGate(P = fit$train.Data.cell$y.Pred.s0,
              x= train_data[,1:7])


## ----results='asis',message=FALSE,fig.width = 5-------------------------------
sessionInfo()

#' Convert fcs files to a data frame
#'
#' A function that convert fcs files to a data frame.
#'
#' @param fcsFiles A vector specifying the location of fcs files (relative to
#'   working directory).
#' @param y A vector containing the clinical outcome of each sample. Must have
#'   the same length as fcsFiles. Null for testing data.
#' @param assay Either "FCM" or "CyTOF" to indicate the type of cytometry data.
#' @param b A positive number used to specify the arcsinh transformation. f(x) =
#'   asinh (b*x) where x is the original value and f(x) is the value after
#'   transformation. The suggested value is 1/150 for flow cytometry (FCM) data
#'   and 1/8 for CyTOF data.
#' @param fileSampleSize An integer specifying the number of events sampled from
#'   each fcs file. If NULL, all the events will be pre-processed and wrote out
#'   to the new fcs files.
#' @param excludeTransformParameters A vector specifying the name of parameters
#'   not to be transformed (left at linear scale).
#' @param compFiles A vector specifying the paths of user supplied compensation
#'   matrix for each fcs file. The matrix must be stored in csv files.
#' @param nameDict A vector used to change marker names.Each element in the
#'   vector is the prefered name of a marker. The name of each element is the
#'   marker name used in the fcs file. For example, a vector
#'   c("CD8b"="CD8","cd8"="CD8") will change "CD8b" and "cd8" into "CD8", making
#'   annotations more consistent.
#' @return Returns a data frame containing the preprocessed cytometry data.
#'   Cells from different fcs files are combined into one flow frame. A new
#'   column, xSample, is introduced to indicate the origin of each cell. The
#'   data frame also includes the clinical outcome y.
#' @importFrom flowCore read.flowSet fsApply keyword logTransform transformList
#'   transform compensate arcsinhTransform colnames
#' @importFrom utils read.csv
#' @examples
#' # Find the table containing fcs file names in CytoDx package
#' path <- system.file("extdata",package="CytoDx")
#' # read the table
#' fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
#' # Specify the path to the cytometry files
#' fn <- file.path(path,fcs_info$fcsName)
#' # Read cytometry files using fcs2DF function
#' train_data <- fcs2DF(fcsFiles=fn,
#'                     y=fcs_info$Label,
#'                     assay="FCM",
#'                     b=1/150,
#'                     excludeTransformParameters=
#'                       c("FSC-A","FSC-W","FSC-H","Time"))
#' @export
fcs2DF <- function(fcsFiles,
                y=NULL,
                assay=c("FCM", "CyTOF"),
                b=1/200,
                fileSampleSize=5000,
                compFiles=NULL,
                nameDict=NULL,
                excludeTransformParameters=
                  c("FSC-A","FSC-W","FSC-H","Time","Cell_length")){

  fcs_param <- NULL
  fcs_names <- gsub(".*/","",fcsFiles)
  excludeTransformParameters <- paste(excludeTransformParameters,collapse="|")

  # 1) identify sample :
  fcsFiles <- as.character(fcsFiles)

  # 2) read the fcs files
  fcs <- flowCore::read.flowSet(fcsFiles,transformation="linearize",alter.names=FALSE,truncate_max_range=FALSE)
  if(!is.null(fileSampleSize)){
    fcs <- flowCore::fsApply(fcs, function(f){
      L <- nrow(f@exprs)
      if(L>fileSampleSize){
        f@exprs <- f@exprs[sample(seq_len(L),fileSampleSize),]
      }
      return(f)
    })
  }

  # 3) compensation and transformation
  if (assay == "FCM") {
    # retreiving fluorescent biomarkers
    w <- which(!grepl(excludeTransformParameters,flowCore::colnames(fcs),ignore.case = TRUE))
    biomarker_vector <- flowCore::colnames(fcs)[w]
    # identify the fcs file format
    version <- flowCore::fsApply(fcs, function(frame) {
      return(flowCore::keyword(frame, "FCSversion")[[1]])
    })
    unique_version <- as.numeric(unique(as.vector(version)))

    # compensation, transformation
    ## if the file version is 2.0, then code will log transform the data
    if (unique_version == 2) {
      trans <- flowCore::logTransform()
      translist <- flowCore::transformList(biomarker_vector,trans)
      fcs <- flowCore::transform(fcs, translist)
    } else if (unique_version >= 3) {
      # check if user have provided the compmatrix
      if(!is.null(compFiles)){
        compList=lapply(compFiles,function(x){
          if(is.na(x)){return(as.matrix(NA))}else{return(as.matrix(read.csv(x,row.names = 1,check.names = FALSE)))}
        })
        for(id in seq_along(fcs)){
          if(!is.na(compList[[id]][1,1])){
            fcs[[id]]@description$SPILL <- compList[[id]]
          }
        }
      }

      ### function first checks if the spill matrix in the flowframe is an actual spill matrix or an identity matrix
      if (is.null(flowCore::keyword(fcs[[1]], "SPILL")[[1]]) == FALSE) {
        check <- flowCore::fsApply(fcs, function(x) {
          result <- isSymmetric(flowCore::keyword(x, "SPILL")[[1]])
        })
        ### if all flowframes have spill matrices then my function
        ### will apply them to each flowframe in the flowset for compensation
        if (unique(check)[[1]] == FALSE) {
          fcs <- flowCore::fsApply(fcs, function(x) {
            new_frame <- flowCore::compensate(x, flowCore::keyword(x, "SPILL")[[1]])
            return(new_frame)
          })
        }
      }

      trans <- flowCore::arcsinhTransform(transformationId="defaultArcsinhTransform",a=0,b=b,c=0)
      translist <- flowCore::transformList(biomarker_vector, trans)
      fcs <- flowCore::transform(fcs, translist)
    }
  } else if (assay == "CyTOF") {
    w <- which(!grepl(excludeTransformParameters,flowCore::colnames(fcs),ignore.case = TRUE))
    biomarker_vector  <-  flowCore::colnames(fcs)[w]

    trans <- flowCore::arcsinhTransform(transformationId = "defaultArcsinhTransform", a = 0, b = b, c = 0)
    translist <- flowCore::transformList(biomarker_vector, trans)
    fcs <- flowCore::transform(fcs, translist)
  }
  fcs <- set2DF(fcs,fcsFiles,y)
  if(!is.null(nameDict)){
    t1 <- colnames(fcs)
    w <- which(t1%in%names(nameDict))
    t1[w] <- nameDict[t1[w]]
    colnames(fcs) <- t1
  }
  return(fcs)
}

#' convert a flowSet to a data frame
#'
#' A function that convert a flowSet to a data frame.
#'
#' @param flowSet A flowSet object
#' @param fcsFiles A vector containing the name of each fcs file included in
#'   flowSet.
#' @param y The clinical outcome each fcs file associated with. Null for testing
#'   data.
#' @return Returns a data frame containing the cytometry data. Cells from
#'   different fcs files are combined into one flow frame. A new column,
#'   xSample, is introduced to indicate the origin of each cell. The data frame
#'   also includes the clinical outcome y.
#' @examples
#' library(flowCore)
#' # Find the table containing fcs file names in CytoDx package
#' path <- system.file("extdata",package="CytoDx")
#' # read the table
#' fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
#' # Specify the path to the cytometry files
#' fn <- file.path(path,fcs_info$fcsName)
#' fSet <- read.flowSet(fn)
#' df <- set2DF(flowSet=fSet,fcsFiles=fn,y = fcs_info$Label)
#' @export


set2DF <- function(flowSet,fcsFiles,y = NULL){
  expr <- flowCore::fsApply(flowSet,function(x){
    v <- flowCore::exprs(x);
    return(v)
  })
  eventN <- flowCore::fsApply(flowSet,function(x){
    v <- flowCore::exprs(x);
    n <- nrow(v)
    return(n)
  })
  Label <- flowCore::fsApply(flowSet,function(x){
    v <- flowCore::pData(flowCore::parameters(x));
    return(v)
  })
  #annotate expr colnames
  channels <- Label[[1]]$name
  antibodies <- Label[[1]]$desc

  antibodies <- vapply(seq_along(antibodies), function(i){
    if(is.na(antibodies[i])|antibodies[i]=="NA"){return(channels[i])}else{antibodies[i]}
  }, character(1))
  colnames(expr)=antibodies
  #add sample_id
  sample_id <- rep(fcsFiles,times = eventN)
  if(is.null(y)){y <- rep(NA,nrow(expr))}else{
    y <- as.data.frame(y)
    y <- y[rep(seq_len(nrow(y)),times = eventN),,drop=FALSE]
  }
  expr <- cbind.data.frame(expr,"xSample"=sample_id,y)
  return(expr)
}

#' Build the CytoDx model
#'
#' A function that builds the CytoDx model.
#'
#' @param x The marker profile of cells pooled from all samples. Each row is a cell, each column is
#'   a marker.
#' @param y The clinical outcomes associated with samples to which cells belong. Length must be equal to nrow(x). For
#'   family="binomial" should be either a factor with two levels, or a
#'   two-column matrix of counts or proportions (the second column is treated as
#'   the target class; for a factor, the last level in alphabetical order is the
#'   target class). For family="multinomial", can be a nc>=2 level factor, or a
#'   matrix with nc columns of counts or proportions. For either "binomial" or
#'   "multinomial", if y is presented as a vector, it will be coerced into a
#'   factor. For family="cox", y should be a two-column matrix with columns
#'   named 'time' and 'status'. The latter is a binary variable, with '1'
#'   indicating death, and '0' indicating right censored. The function Surv() in
#'   package survival produces such a matrix. For family="mgaussian", y is a
#'   matrix of quantitative responses.
#' @param xSample A vector specifying which sample each cell belongs to. Length
#'   must equal to nrow(x).
#' @param family Response type. Must be one of the following:
#'   "gaussian","binomial","poisson","multinomial","cox","mgaussian"
#' @param type1 Type of first level prediction. Type of prediction required.
#'   Type "link" gives the linear predictors for "binomial", "multinomial",
#'   "poisson" or "cox" models; for "gaussian" models it gives the fitted
#'   values. Type "response" gives the fitted probabilities for "binomial" or
#'   "multinomial", fitted mean for "poisson" and the fitted relative-risk for
#'   "cox"; for "gaussian" type "response" is equivalent to type "link".
#' @param type2 Type of second level prediction.
#' @param parallelCore The number of core to be used. Only used when reg is
#'   TRUE.
#' @param reg If elestic net regularization will be used.
#' @param ... Other parameters to be passed into the glmnet or the cv.glmnet
#'   function in the glmnet package.
#' @return Returns a list. train.Data.cell contains the trainig data and the
#'   predicted y for the training data at the cell level. model.cell contains the
#'   cell stage statistical model. Data.sample contains the trainig data and the
#'   predicted y for the training data at the sample level. model.sample contains the
#'   sample stage statistical model. family specifies the regression type.
#'   method specifies the type of learning method. type.cell is the type of cell
#'   level prediction. type.sample is the type of sample level prediction.
#' @examples
#' # Find the table containing fcs file names in CytoDx package
#' path <- system.file("extdata",package="CytoDx")
#' # read the table
#' fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
#' # Specify the path to the cytometry files
#' fn <- file.path(path,fcs_info$fcsName)
#' # Read cytometry files using fcs2DF function
#' train_data <- fcs2DF(fcsFiles=fn,
#'                     y=fcs_info$Label,
#'                     assay="FCM",
#'                     b=1/150,
#'                     excludeTransformParameters=
#'                       c("FSC-A","FSC-W","FSC-H","Time"))
#' # build the model
#' fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
#'                 y=train_data$y,
#'                 xSample = train_data$xSample,
#'                 reg=FALSE,
#'                 family="binomial")
#' # check accuracy for training data
#' pred <- CytoDx.pred(fit,
#'                    xNew=as.matrix(train_data[,1:7]),
#'                    xSampleNew=train_data$xSample)
#'
#' boxplot(pred$xNew.Pred.sample$y.Pred.s0~
#'           fcs_info$Label)
#'
#' @importFrom doParallel registerDoParallel
#' @importFrom glmnet cv.glmnet glmnet
#' @importFrom stats predict
#' @importFrom dplyr %>% group_by summarise_all left_join
#' @import grDevices
#' @import graphics
#' @export

CytoDx.fit <-  function(x,y,xSample,
                     family = c("gaussian","binomial","poisson","multinomial","cox","mgaussian"),
                     type1="response",type2="response",
                          parallelCore=1,reg=FALSE,...){
  stopifnot(length(xSample) == nrow(x))

  stopifnot(nrow(as.matrix(y)) == nrow(x))


  family <- match.arg(family)
  y <- as.matrix(y)
  xSample <- as.character(xSample)
  if(parallelCore>1){doParallel::registerDoParallel(parallelCore)}


  if(reg==TRUE){
    fit <- glmnet::cv.glmnet(x=x,y=y,family=family,
                             parallel=(parallelCore>1),...)
  }else{
    fit <- glmnet::glmnet(x=x,y=y,family=family,lambda = 0,...)
  }

  preds <- predict(fit,newx = x,type=type1)
  train.Data <- cbind.data.frame(xSample,y,preds)
  y <- as.matrix(y)
  colnames(train.Data) <- c("sample",paste0("y",seq_len(ncol(y)),".Truth"),
                         paste0("y",".Pred.",colnames(preds)))

  SP.train <- train.Data%>%group_by(sample)%>%summarise_all(meanUnique)


  y <- SP.train[,grep(".Truth",colnames(SP.train))]
  y <- as.matrix(y)
  if(family=="cox"){colnames(y)=c("time","status")}
  x <- SP.train[,grep(".Pred",colnames(SP.train))]
  x <- as.matrix(cbind("constant"=1,x))
  fit2 <- glmnet::glmnet(x=x,y=y,family=family,lambda=0)

  train.Data2 <-  predict(fit2,x,type=type2)
  SP.train[,grep(".Pred",colnames(SP.train))] <- as.data.frame(train.Data2)
  SP.train=as.data.frame(SP.train)
  colnames(SP.train) <- colnames(train.Data)
  row.names(SP.train) <- SP.train$sample
  SP.train  <- SP.train[unique(xSample),]
  return(list("train.Data.cell"=train.Data,"model.cell"=fit,"train.Data.sample"=SP.train,
              "model.sample"=fit2,"family"=family,"type.cell"=type1,"type.sample"=type2))
}

#' Percentile rank transformation of the data
#'
#' A function that performs the rank transformation of the data.
#'
#' @param x A data frame containing the pooled data from fcs files. Each row is
#'   a cell, each column is a marker.
#' @param xSample A vector specifying which sample each cell belongs to. Length
#'   must equal to nrow(x).
#' @return Returns data frame containing rank transformed data.
#' @importFrom dplyr mutate_all
#' @examples
#' x <- pRank(x=iris[,1:4],xSample=iris$Species)
#' @export


pRank <- function(x,xSample){

  stopifnot(length(xSample) == nrow(x))

  x <- cbind.data.frame(x,"xSample"=xSample)%>%
    dplyr::group_by(xSample)%>%
    dplyr::mutate_all(.funs=rank.ub.average)%>%
    as.data.frame()%>%
    dplyr::select(-xSample)
  return(x)
}

#' Use decision tree to find a group of cells that are associated with clinical
#' outcome.
#'
#' A function that sse decision tree to find a group of cells that are
#' associated with clinical outcome.
#'
#' @param P The predicted association of each cell with a clinical outcome.
#' @param x The marker profile of each cell. Each row is a cell, each column is
#'   a marker. Must have length(P) rows.
#' @param ... Other parameters to be passed into the rpart function
#' @return Returns a object created by rpart function. Also plots a graph of
#'   decision tree.
#' @examples
#' # Find the table containing fcs file names in CytoDx package
#' path=system.file("extdata",package="CytoDx")
#' # read the table
#' fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
#' # Specify the path to the cytometry files
#' fn <- file.path(path,fcs_info$fcsName)
#' # Read cytometry files using fcs2DF function
#' train_data <- fcs2DF(fcsFiles=fn,
#'                     y=fcs_info$Label,
#'                     assay="FCM",
#'                     b=1/150,
#'                     excludeTransformParameters=
#'                       c("FSC-A","FSC-W","FSC-H","Time"))
#' # build the model
#' fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
#'                 y=train_data$y,
#'                 xSample = train_data$xSample,
#'                 reg=FALSE,
#'                 family="binomial")
#' # check accuracy for training data
#' pred <- CytoDx.pred(fit,
#'                    xNew=as.matrix(train_data[,1:7]),
#'                    xSampleNew=train_data$xSample)
#'
#' boxplot(pred$xNew.Pred.sample$y.Pred.s0~
#'           fcs_info$Label)
#'
#' # Find the associated population using treeGate
#' TG <- treeGate(P = fit$train.Data.cell$y.Pred.s0,
#'               x= train_data[,1:7])
#' @importFrom rpart rpart rpart.control
#' @importFrom rpart.plot rpart.plot
#' @export
treeGate <- function(P,x,...){
  colnames(x) <- gsub("[^[:alnum:]]", "_",colnames(x))
  x <- cbind.data.frame("P"=P,x)
  fit <- rpart(P~.,method="anova", data=x,...)
  rpart.plot(fit)
  return(fit)
}

#' Calulate mean or take unique elements of a vector
#'
#' A function that calulate mean or take unique elements of a vector.
#'
#' @param x a vector
#' @return If x is numeric, returns the mean. Otherwise, returns the unique elements of x.
#' @examples
#' x <- 1:5
#' meanUnique(x)
#' x=c("a","a","b")
#' meanUnique(x)
#' @export

meanUnique <- function(x){
  if(is.numeric(x)){
    return(mean(x,na.rm = TRUE))
  }else(return(unique(x)))
}

#' Percentile rank transformation of a vector
#'
#' A function that performs the Percentile rank transformation of a vector
#'
#' @param x A numeric vector.
#' @return Returns the percentile rank of each element.
#' @examples
#' rank.ub.average(1:10)
#' @export
rank.ub.average <- function(x){
  n <- length(unique(x))
  N <- length(x)
  if(n==1){x <- rep(NA,N)}
  if(n>1){x  <- (rank(x,ties.method="average")-1)/(N-1)*100}
  return(x)
}

#' Make prediction using the CytoDx model
#'
#' A function that makes prediction using the CytoDx model.
#'
#' @param fit The two stage statistical model. Must be the object returned by CytoDx.fit.
#' @param xNew The marker profile of cells pooled from all new samples. Each row is a cell, each column is
#'   a marker.
#' @param xSampleNew A vector specifying which sample each cell belongs to. Length
#'   must equal to nrow(xNew).
#' @return Returns a list. xNew.Pred1 contains the predicted y for the new data at the cell level.
#'   xNew.Pred2 contains the predicted y for the new data at the sample level.
#' @examples
#' # Find the table containing fcs file names in CytoDx package
#' path <- system.file("extdata",package="CytoDx")
#' # read the table
#' fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
#' # Specify the path to the cytometry files
#' fn <- file.path(path,fcs_info$fcsName)
# Read cytometry files using fcs2DF function
#' train_data <- fcs2DF(fcsFiles=fn,
#'                     y=fcs_info$Label,
#'                     assay="FCM",
#'                     b=1/150,
#'                     excludeTransformParameters=
#'                       c("FSC-A","FSC-W","FSC-H","Time"))
#' # build the model
#' fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
#'                 y=train_data$y,
#'                 xSample = train_data$xSample,
#'                 reg=FALSE,
#'                 family="binomial")
#' # check accuracy for training data
#' pred <- CytoDx.pred(fit,
#'                    xNew=as.matrix(train_data[,1:7]),
#'                    xSampleNew=train_data$xSample)
#'
#' boxplot(pred$xNew.Pred.sample$y.Pred.s0~
#'           fcs_info$Label)
#'
#' @importFrom dplyr %>% group_by summarise_all left_join
#' @importFrom glmnet glmnet
#' @importFrom stats predict
#' @export

CytoDx.pred <- function(fit,xNew,xSampleNew){

  stopifnot(length(xSampleNew) == nrow(xNew))

  xSampleNew <- as.character(xSampleNew)

  preds <- predict(fit$model.cell,xNew,type=fit$type.cell)


  xNew.Pred <- cbind.data.frame("sample"=xSampleNew,preds)
  colnames(xNew.Pred) <- c("sample",paste0("y",".Pred.",colnames(preds)))
  SP.new <- xNew.Pred%>%group_by(sample)%>%summarise_all(mean)

  xNew <- SP.new[,grep(".Pred",colnames(SP.new))]%>%as.matrix()
  xNew <- cbind("constant"=1,xNew)
  xNew.Pred2 <- predict(fit$model.sample,xNew,type=fit$type.sample)
  SP.new[,grep(".Pred",colnames(SP.new))] <- as.data.frame(xNew.Pred2)

  SP.new <- as.data.frame(SP.new)
  colnames(SP.new) <- colnames(xNew.Pred)
  row.names(SP.new) <- SP.new$sample

  SP.new  <-  SP.new[unique(xSampleNew),]
  return(list("xNew.Pred.cell"=xNew.Pred,
              "xNew.Pred.sample"=SP.new))

}

---
title: "Introduction to CytoDx"
author: "Zicheng Hu"
date: "`r Sys.Date()`"
output:
  pdf_document:
    toc: true
    toc_depth: 3

vignette: >
  %\VignetteIndexEntry{Introduction to CytoDx}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

\newpage

## Introduction
CytoDx is a method that predicts clinical outcomes using single cell data without the need of cell gating. It first predicts the association between each cell and the outcome using a linear statistical model (Figure 1). The cell level predictions are then averaged within each sample to represent the sample level predictor. A second model is used to make prediction at the sample level (Figure 1). Compare to traditional gating based methods, CytoDX have multiple advantages. 

1.	Robustness. CytoDx is able to robustly predict clinical outcomes using data acquired by different cytometry platforms in different research institutes.
2.	Interpretability. CytoDx identifies cell markers and cell subsets that are most associated with the clinical outcome, allowing researchers to interpret the result easily. 
3.	Simplicity. CytoDx associates cytometry data with clinical features without any cell gating steps, therefore simplifying the diagnostic process in clinical settings.  

In section 2, we demonstrate how to install CytoDx.

In section 3, we show an example where CytoDx is used to diagnose acute myeloid leukemia (AML). In addition to perform diagnosis, we also show that CytoDx can be used to identify the cell subsets that are associated with AML. 

```{r, out.width = "500px",echo=FALSE}
knitr::include_graphics("tssm_intro.jpg")
```

## Installation
You can install the stable version of CytoDx from Bioconductor:
```{r, eval=FALSE}
BiocManager::install("CytoDx")                                                                                                                                                                                                                
```

You can also install the latest develop version of CytoDx from github:
```{r, eval=FALSE}
devtools::install_github("hzc363/CytoDx")                                                                                                                                                                                                                    
```

## Example: diagnosing AML using flow cytometry

In this example, we build a CytoDx model to diagnose acute myeloid leukemia (AML) using flow cytometry data. We train the model using data from 5 AML patients and 5 controls and test the performance in a test dataset. 

### Step 1: Prepare data
The CytoDx R package contains the fcs files and the ground truth (AML or normal) that are needed for our example. We first load the ground truth.

```{r, results='asis',message=FALSE}
library(CytoDx)

# Find data in CytoDx package
path <- system.file("extdata",package="CytoDx")

# read the ground truth
fcs_info  <- read.csv(file.path(path,"fcs_info.csv"))

# print out the ground truth
knitr::kable(fcs_info)
```

We then read the cytometry data for training samples using the fcs2DF function.
```{r, results='asis',message=FALSE}
# Find the training data
train_info <- subset(fcs_info,fcs_info$dataset=="train")

# Specify the path to the cytometry files
fn <- file.path(path,train_info$fcsName)

# Read cytometry files using fcs2DF function
train_data <- fcs2DF(fcsFiles=fn,
                    y=train_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
```

The CytoDx is flexible to data transformations. It can be applied to rank transformed data to reduce batch effects. Here, we transform the original data to rank data.
```{r, results='asis',message=FALSE}
# Perfroms rank transformation
x_train <- pRank(x=train_data[,1:7],xSample=train_data$xSample)

# Convert data frame into matrix. Here we included the 2-way interactions.
x_train <- model.matrix(~.*.,x_train)
```

### Step 2: Build CytoDx model
We use training data to build a predictive model. 
```{r, results='asis',message=FALSE,warning=FALSE}
# Build predictive model using the CytoDx.fit function
fit <- CytoDx.fit(x=x_train,
               y=(train_data$y=="aml"),
               xSample=train_data$xSample,
               family = "binomial",
               reg = FALSE)
```

### Step 3: Predict AML using testing data
We first load and rank transform the test data.
```{r, results='asis',message=FALSE}
# Find testing data
test_info <- subset(fcs_info,fcs_info$dataset=="test")

# Specify the path to cytometry files
fn <- file.path(path,test_info$fcsName)

# Read cytometry files using fcs2DF function
test_data <- fcs2DF(fcsFiles=fn,
                    y=NULL,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
# Perfroms rank transformation
x_test <- pRank(x=test_data[,1:7],xSample=test_data$xSample)

# Convert data frame into matrix. Here we included the 2-way interactions.
x_test <- model.matrix(~.*.,x_test)
```

We use the built CytoDx model to predict AML.
```{r, results='asis',message=FALSE}
# Predict AML using CytoDx.ped function
pred <- CytoDx.pred(fit,xNew=x_test,xSampleNew=test_data$xSample)
```

We plot the prediction. In this example, CytoDx classifies the sample into AML and normal perfectly. 
```{r, results='asis',message=FALSE,fig.width = 5}
# Cmbine prediction and truth
result <- data.frame("Truth"=test_info$Label,
                    "Prob"=pred$xNew.Pred.sample$y.Pred.s0)

# Plot the prediction
stripchart(result$Prob~result$Truth, jitter = 0.1,
           vertical = TRUE, method = "jitter", pch = 20,
           xlab="Truth",ylab="Predicted Prob of AML")
```

### Step 4: Find cell subsets associated with AML
We use a decision tree to find cell subsets that are associated the AML. In this step, the original cytometry data should be used, rather than the ranked data. 
```{r, results='asis',message=FALSE,fig.width = 5}
# Use decision tree to find the cell subsets that are associated the AML.
TG <- treeGate(P = fit$train.Data.cell$y.Pred.s0,
              x= train_data[,1:7])

```

### Session Infomation
```{r, results='asis',message=FALSE,fig.width = 5}
sessionInfo()

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank.ub.average.R
\name{rank.ub.average}
\alias{rank.ub.average}
\title{Percentile rank transformation of a vector}
\usage{
rank.ub.average(x)
}
\arguments{
\item{x}{A numeric vector.}
}
\value{
Returns the percentile rank of each element.
}
\description{
A function that performs the Percentile rank transformation of a vector
}
\examples{
rank.ub.average(1:10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcs2DF.R
\name{fcs2DF}
\alias{fcs2DF}
\title{Convert fcs files to a data frame}
\usage{
fcs2DF(fcsFiles, y = NULL, assay = c("FCM", "CyTOF"), b = 1/200,
  fileSampleSize = 5000, compFiles = NULL, nameDict = NULL,
  excludeTransformParameters = c("FSC-A", "FSC-W", "FSC-H", "Time",
  "Cell_length"))
}
\arguments{
\item{fcsFiles}{A vector specifying the location of fcs files (relative to
working directory).}

\item{y}{A vector containing the clinical outcome of each sample. Must have
the same length as fcsFiles. Null for testing data.}

\item{assay}{Either "FCM" or "CyTOF" to indicate the type of cytometry data.}

\item{b}{A positive number used to specify the arcsinh transformation. f(x) =
asinh (b*x) where x is the original value and f(x) is the value after
transformation. The suggested value is 1/150 for flow cytometry (FCM) data
and 1/8 for CyTOF data.}

\item{fileSampleSize}{An integer specifying the number of events sampled from
each fcs file. If NULL, all the events will be pre-processed and wrote out
to the new fcs files.}

\item{compFiles}{A vector specifying the paths of user supplied compensation
matrix for each fcs file. The matrix must be stored in csv files.}

\item{nameDict}{A vector used to change marker names.Each element in the
vector is the prefered name of a marker. The name of each element is the
marker name used in the fcs file. For example, a vector
c("CD8b"="CD8","cd8"="CD8") will change "CD8b" and "cd8" into "CD8", making
annotations more consistent.}

\item{excludeTransformParameters}{A vector specifying the name of parameters
not to be transformed (left at linear scale).}
}
\value{
Returns a data frame containing the preprocessed cytometry data.
  Cells from different fcs files are combined into one flow frame. A new
  column, xSample, is introduced to indicate the origin of each cell. The
  data frame also includes the clinical outcome y.
}
\description{
A function that convert fcs files to a data frame.
}
\examples{
# Find the table containing fcs file names in CytoDx package
path <- system.file("extdata",package="CytoDx")
# read the table
fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
# Specify the path to the cytometry files
fn <- file.path(path,fcs_info$fcsName)
# Read cytometry files using fcs2DF function
train_data <- fcs2DF(fcsFiles=fn,
                    y=fcs_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanUnique.R
\name{meanUnique}
\alias{meanUnique}
\title{Calulate mean or take unique elements of a vector}
\usage{
meanUnique(x)
}
\arguments{
\item{x}{a vector}
}
\value{
If x is numeric, returns the mean. Otherwise, returns the unique elements of x.
}
\description{
A function that calulate mean or take unique elements of a vector.
}
\examples{
x <- 1:5
meanUnique(x)
x=c("a","a","b")
meanUnique(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pRank.R
\name{pRank}
\alias{pRank}
\title{Percentile rank transformation of the data}
\usage{
pRank(x, xSample)
}
\arguments{
\item{x}{A data frame containing the pooled data from fcs files. Each row is
a cell, each column is a marker.}

\item{xSample}{A vector specifying which sample each cell belongs to. Length
must equal to nrow(x).}
}
\value{
Returns data frame containing rank transformed data.
}
\description{
A function that performs the rank transformation of the data.
}
\examples{
x <- pRank(x=iris[,1:4],xSample=iris$Species)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set2DF.R
\name{set2DF}
\alias{set2DF}
\title{convert a flowSet to a data frame}
\usage{
set2DF(flowSet, fcsFiles, y = NULL)
}
\arguments{
\item{flowSet}{A flowSet object}

\item{fcsFiles}{A vector containing the name of each fcs file included in
flowSet.}

\item{y}{The clinical outcome each fcs file associated with. Null for testing
data.}
}
\value{
Returns a data frame containing the cytometry data. Cells from
  different fcs files are combined into one flow frame. A new column,
  xSample, is introduced to indicate the origin of each cell. The data frame
  also includes the clinical outcome y.
}
\description{
A function that convert a flowSet to a data frame.
}
\examples{
library(flowCore)
# Find the table containing fcs file names in CytoDx package
path <- system.file("extdata",package="CytoDx")
# read the table
fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
# Specify the path to the cytometry files
fn <- file.path(path,fcs_info$fcsName)
fSet <- read.flowSet(fn)
df <- set2DF(flowSet=fSet,fcsFiles=fn,y = fcs_info$Label)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treeGate.R
\name{treeGate}
\alias{treeGate}
\title{Use decision tree to find a group of cells that are associated with clinical
outcome.}
\usage{
treeGate(P, x, ...)
}
\arguments{
\item{P}{The predicted association of each cell with a clinical outcome.}

\item{x}{The marker profile of each cell. Each row is a cell, each column is
a marker. Must have length(P) rows.}

\item{...}{Other parameters to be passed into the rpart function}
}
\value{
Returns a object created by rpart function. Also plots a graph of
  decision tree.
}
\description{
A function that sse decision tree to find a group of cells that are
associated with clinical outcome.
}
\examples{
# Find the table containing fcs file names in CytoDx package
path=system.file("extdata",package="CytoDx")
# read the table
fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
# Specify the path to the cytometry files
fn <- file.path(path,fcs_info$fcsName)
# Read cytometry files using fcs2DF function
train_data <- fcs2DF(fcsFiles=fn,
                    y=fcs_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
# build the model
fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
                y=train_data$y,
                xSample = train_data$xSample,
                reg=FALSE,
                family="binomial")
# check accuracy for training data
pred <- CytoDx.pred(fit,
                   xNew=as.matrix(train_data[,1:7]),
                   xSampleNew=train_data$xSample)

boxplot(pred$xNew.Pred.sample$y.Pred.s0~
          fcs_info$Label)

# Find the associated population using treeGate
TG <- treeGate(P = fit$train.Data.cell$y.Pred.s0,
              x= train_data[,1:7])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CytoDx.fit.R
\name{CytoDx.fit}
\alias{CytoDx.fit}
\title{Build the CytoDx model}
\usage{
CytoDx.fit(x, y, xSample, family = c("gaussian", "binomial", "poisson",
  "multinomial", "cox", "mgaussian"), type1 = "response",
  type2 = "response", parallelCore = 1, reg = FALSE, ...)
}
\arguments{
\item{x}{The marker profile of cells pooled from all samples. Each row is a cell, each column is
a marker.}

\item{y}{The clinical outcomes associated with samples to which cells belong. Length must be equal to nrow(x). For
family="binomial" should be either a factor with two levels, or a
two-column matrix of counts or proportions (the second column is treated as
the target class; for a factor, the last level in alphabetical order is the
target class). For family="multinomial", can be a nc>=2 level factor, or a
matrix with nc columns of counts or proportions. For either "binomial" or
"multinomial", if y is presented as a vector, it will be coerced into a
factor. For family="cox", y should be a two-column matrix with columns
named 'time' and 'status'. The latter is a binary variable, with '1'
indicating death, and '0' indicating right censored. The function Surv() in
package survival produces such a matrix. For family="mgaussian", y is a
matrix of quantitative responses.}

\item{xSample}{A vector specifying which sample each cell belongs to. Length
must equal to nrow(x).}

\item{family}{Response type. Must be one of the following:
"gaussian","binomial","poisson","multinomial","cox","mgaussian"}

\item{type1}{Type of first level prediction. Type of prediction required.
Type "link" gives the linear predictors for "binomial", "multinomial",
"poisson" or "cox" models; for "gaussian" models it gives the fitted
values. Type "response" gives the fitted probabilities for "binomial" or
"multinomial", fitted mean for "poisson" and the fitted relative-risk for
"cox"; for "gaussian" type "response" is equivalent to type "link".}

\item{type2}{Type of second level prediction.}

\item{parallelCore}{The number of core to be used. Only used when reg is
TRUE.}

\item{reg}{If elestic net regularization will be used.}

\item{...}{Other parameters to be passed into the glmnet or the cv.glmnet
function in the glmnet package.}
}
\value{
Returns a list. train.Data.cell contains the trainig data and the
  predicted y for the training data at the cell level. model.cell contains the
  cell stage statistical model. Data.sample contains the trainig data and the
  predicted y for the training data at the sample level. model.sample contains the
  sample stage statistical model. family specifies the regression type.
  method specifies the type of learning method. type.cell is the type of cell
  level prediction. type.sample is the type of sample level prediction.
}
\description{
A function that builds the CytoDx model.
}
\examples{
# Find the table containing fcs file names in CytoDx package
path <- system.file("extdata",package="CytoDx")
# read the table
fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
# Specify the path to the cytometry files
fn <- file.path(path,fcs_info$fcsName)
# Read cytometry files using fcs2DF function
train_data <- fcs2DF(fcsFiles=fn,
                    y=fcs_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
# build the model
fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
                y=train_data$y,
                xSample = train_data$xSample,
                reg=FALSE,
                family="binomial")
# check accuracy for training data
pred <- CytoDx.pred(fit,
                   xNew=as.matrix(train_data[,1:7]),
                   xSampleNew=train_data$xSample)

boxplot(pred$xNew.Pred.sample$y.Pred.s0~
          fcs_info$Label)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CytoDx.pred.R
\name{CytoDx.pred}
\alias{CytoDx.pred}
\title{Make prediction using the CytoDx model}
\usage{
CytoDx.pred(fit, xNew, xSampleNew)
}
\arguments{
\item{fit}{The two stage statistical model. Must be the object returned by CytoDx.fit.}

\item{xNew}{The marker profile of cells pooled from all new samples. Each row is a cell, each column is
a marker.}

\item{xSampleNew}{A vector specifying which sample each cell belongs to. Length
must equal to nrow(xNew).}
}
\value{
Returns a list. xNew.Pred1 contains the predicted y for the new data at the cell level.
  xNew.Pred2 contains the predicted y for the new data at the sample level.
}
\description{
A function that makes prediction using the CytoDx model.
}
\examples{
# Find the table containing fcs file names in CytoDx package
path <- system.file("extdata",package="CytoDx")
# read the table
fcs_info <- read.csv(file.path(path,"fcs_info.csv"))
# Specify the path to the cytometry files
fn <- file.path(path,fcs_info$fcsName)
train_data <- fcs2DF(fcsFiles=fn,
                    y=fcs_info$Label,
                    assay="FCM",
                    b=1/150,
                    excludeTransformParameters=
                      c("FSC-A","FSC-W","FSC-H","Time"))
# build the model
fit <- CytoDx.fit(x=as.matrix(train_data[,1:7]),
                y=train_data$y,
                xSample = train_data$xSample,
                reg=FALSE,
                family="binomial")
# check accuracy for training data
pred <- CytoDx.pred(fit,
                   xNew=as.matrix(train_data[,1:7]),
                   xSampleNew=train_data$xSample)

boxplot(pred$xNew.Pred.sample$y.Pred.s0~
          fcs_info$Label)

}
